﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import Checkbox from "@react-native-community/checkbox"
import { observer } from "mobx-react"
import React, { useEffect, useState } from "react"
import { Keyboard, Text, TouchableOpacity, View } from "react-native"
import Modal from 'react-native-modal'
import { SafeAreaView } from 'react-native-safe-area-context'
import { store } from "../../../../main-store"
import { Separator } from "../../../Separator"
import { FilterLmTypes } from './filter-lm-types'
import { FilterMinRating } from './filter-min-rating'

// TODO: change filters to server side processing
/**
 * Modal panel that displays filter controls for the landmarks
 */
export const FilterPanel: React.FC = observer(() => {
    /**
     * These three state variables are copies of their mapState counterparts, and are kept local to this component. They hold the values the sets user on this panel. 
     * They are kept local until "Apply filters is pressed", which will copy their values into their mapState counterparts and trigger the filtering
     * These are used locally instead of the mapState values in order to prevent the entire Map component from re-rendering every time one of these values is changed, 
     * greatly improving performance of this component
    */ 

    const [localMinRating, setLocalMinRating] = useState<number>(store.filters.current["min-rating"] as number)
    const [localFilterTypes, setLocalFilterTypes] = useState<number[]>(store.filters.current["landmark-types"] as number[])
    const [localOwned, toggleLocalOwned] = useState<boolean>(store.filters.current['only-owned'] as boolean)

    /**
     * Resets only local filter values
     */
    const cancelChanges = () => {
        setLocalFilterTypes([])
        setLocalMinRating(0)
        toggleLocalOwned(false)
    }

    /**
     * Sets the map's filter state to the value of the local filter state, clears the local state, closes the panel and refetches the landmarks.
     */
    const applyChanges = () => {
        store.filters.setFilters([
            {
                type: 'landmark-types',
                value: localFilterTypes
            },
            {
                type: 'min-rating',
                value: localMinRating
            },
            {
                type: 'only-owned',
                value: localOwned
            }
        ])
        setLocalMinRating(0)
        setLocalFilterTypes([])
        toggleLocalOwned(false)
    }   

    // useEffects that update local settings based on parent changes (e.g. deleting a chip)
    useEffect(() => {
        setLocalMinRating(store.filters.current["min-rating"] as number)
        setLocalFilterTypes(store.filters.current["landmark-types"] as number[])
        toggleLocalOwned(store.filters.current['only-owned'] as boolean)
    }, [store.filters.current])

    const OwnedFilter: React.FC = () => {
        return (
            <View style={{flexDirection: 'row', paddingTop: 10, paddingBottom: 15}} >
                <Text>Only show my landmarks:</Text>
                <Checkbox value={localOwned} onValueChange={() => toggleLocalOwned(!localOwned)} boxType="square" style={{marginLeft: 10, width: 20, height: 20}} />
            </View>
        )
    }

    const FilterButtons: React.FC = () => {
        return (
            <View style={{marginTop: 20, flexDirection: 'row', alignSelf: 'flex-end'}}>
                <TouchableOpacity style={{marginRight: 40}} onPress={() => store.filters.clearFilters()}>
                    <Text>Reset</Text>
                </TouchableOpacity>
                <TouchableOpacity onPress={() => applyChanges()}>
                    <Text>Apply</Text>
                </TouchableOpacity>
            </View>
        )
    }

    return (
        <Modal
            useNativeDriver={true}
            useNativeDriverForBackdrop={true}
            isVisible={store.filters.panelVisible} 
            animationIn={'slideInDown'} 
            animationOut={'slideOutUp'}
            style={{justifyContent: 'flex-start', height: '100%', width: '100%', margin: 0}}
            swipeDirection={'up'}
            onBackdropPress={() => {
                if (store.global.keyboardOpen) {
                    Keyboard.dismiss()
                }
                else {
                    cancelChanges();
                    store.filters.closePanel();
                }
            }} >
                <SafeAreaView style={{padding: 20, backgroundColor: 'white'}}>
                    <FilterMinRating 
                        localMinRating={localMinRating} 
                        setLocalMinRating={setLocalMinRating} 
                        cancelChanges={cancelChanges}/>
                    <FilterLmTypes localFilterTypes={localFilterTypes} setLocalFilterTypes={setLocalFilterTypes} />
                    <OwnedFilter />
                    <Separator color='grey' />   
                    <FilterButtons />
                </SafeAreaView> 
            </Modal>
        
    )
})