﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { FontAwesome } from "@expo/vector-icons";
import { useNavigation } from "@react-navigation/native";
import React, { useRef, useState } from "react";
import { FlatList, Keyboard, ListRenderItem, StyleSheet, Text, TextInput, View } from "react-native";
import { useOwnedProfile } from "../../../../api/profile";
import { store } from "../../../../main-store";
import { useAuth } from "../../../../state/external/auth-provider";
import { LMComment } from "../../../../types";
import { SecondaryButton } from "../../../Buttons";
import { CommentView } from "./comments-view";
import { useSelectedLandmarkContext } from "./selected-landmark-panel.context";

/**
 * Renders all [comments]{@link LMComment} associated with the {@linkcode selectedLandmark} as items for the [FlatList]{@link https://reactnative.dev/docs/flatlist} in this component.
*/
export const CommentsContainer: React.FC = (props) => {
    const [focusedCommentId, focusComment] = useState<string>();
    const [newComment, setNewComment] = useState<string>("");
    const [commentBeingEdited, setCommentBeingEdited] = useState<LMComment>();
    const {commentsQuery, commentListRef, mainScrollRef, editCommentMutation, addCommentMutation, landmarkQuery} = useSelectedLandmarkContext();
    const {profile} = useOwnedProfile()

    const navigation = useNavigation();
    
    const {accessToken, userId} = useAuth()

    /**
     * Set a comment to be the comment currently edited. 
     */
     const startEditingComment = (comment: LMComment) => {
        setCommentBeingEdited(comment);
        commentTextInputRef.current.focus();
    }

    const addComment = async () => {
        if (newComment) {
            addCommentMutation.mutateAsync({
                content: newComment,
                landmark: landmarkQuery.data?.landmark.id,
                poster: userId,
                poster_name: profile.username,
                id: undefined,
                edited: false
            })
        }
    }

    const commentTextInputRef = useRef<TextInput>();

    /**
     * Flatlist render item method for each comment. 
     * @param 
     * @returns 
     */
    const renderComment: ListRenderItem<LMComment> = ({item, index}) => {
        const selected = item.id == focusedCommentId;
        const belowSelected = commentsQuery.data[index - 1]?.id == focusedCommentId;
        const latestComment = index == 0;
        return (
            <View>
                {/* Some conditional rendering to make the borders look nice */}
                {latestComment ? null : <View style={[selected || belowSelected ? {marginVertical: 0} : {marginHorizontal: 10}, {height: 1, borderBottomWidth: 1, borderColor: 'lightgray'}]}></View> }
                <CommentView comment={item} selected={selected} focusComment={focusComment} startEditingComment={setCommentBeingEdited}/>
            </View>
        );
    };

    /**
     * Simple check to see if the landmark has any comments
     */
    const hasComments = () => {
        return commentsQuery.data?.length > 0
    }

    return (  
    <View style={[styles.commentContainer, {borderBottomWidth: 1, borderColor: 'lightgray', paddingBottom: 20, marginBottom: 10}]}>
        {hasComments() ? 
        <>
            <Text style={{color: 'white', marginBottom: 20}}>Comments: </Text>
            <FlatList<LMComment>
                nestedScrollEnabled={true}
                ref={commentListRef}
                keyExtractor={i => i.id}
                data={commentsQuery.data} 
                extraData={focusedCommentId}
                renderItem={renderComment}
                style={{backgroundColor: 'white'}}
                getItemLayout={(data, index) => ({length: commentsQuery.data.length, offset: commentsQuery.data.length * index, index})}/>
        </> : 
        <Text style={{marginVertical: 20, color: 'white'}}>There are no comments on this landmark</Text> }
        {accessToken ?
        <>
        <View style={{height: 1, borderBottomWidth: 1, borderColor: 'lightgray'}}></View>
        <View style={{flexDirection: 'row', backgroundColor: 'white', paddingRight: 15}}>
            <TextInput 
                ref={commentTextInputRef}
                placeholder="Add comment..." 
                onChangeText={commentBeingEdited ? value => setCommentBeingEdited({...commentBeingEdited, content: value}) : value => setNewComment(value)} 
                value={commentBeingEdited ? commentBeingEdited.content : newComment}
                placeholderTextColor="gray" 
                style={{padding: 15, paddingTop: 15, flex: 5}}
                onFocus={() => mainScrollRef.current.scrollToEnd()}
                multiline={true} />
            {newComment || commentBeingEdited ? 
            <View style={{alignItems: 'center', justifyContent: 'space-between', flexDirection: 'row', flex: 1}}>
                <FontAwesome name="times" size={20} color="lightgray" style={{ backgroundColor: 'white',  }} onPress={Keyboard.dismiss} />
                {commentBeingEdited ?
                <FontAwesome name="check" size={20} style={{ marginLeft: 15, marginRight: 15, backgroundColor: 'white', padding: 'auto'}} onPress={async () => editCommentMutation.mutateAsync(commentBeingEdited)} /> :
                <FontAwesome name="paper-plane" size={20} style={{ backgroundColor: 'white', padding: 'auto'}} onPress={async () => await addComment()} /> }
            </View> : null}
        </View>
        </> : 
        <View>
            <SecondaryButton text="Login to add comments" onPress={() => {store.selectedLm.close(); navigation.navigate("Account" as never)}} style={{marginBottom: 20}}/>
        </View>}
    </View>)
}

const styles = StyleSheet.create({
    input: {
        padding: 5,
        color: 'black'
    },
    commentContainer: {
        flex: 5,
        marginBottom: 40,
    },
})