﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { FontAwesome } from "@expo/vector-icons";
import { useNavigation } from "@react-navigation/native";
import { observer } from "mobx-react";
import React from "react";
import { Alert, StyleSheet, Text, TouchableOpacity, View } from "react-native";
import { store } from "../../../../main-store";
import { useAuth } from "../../../../state/external/auth-provider";
import { useSelectedLandmarkContext } from "./selected-landmark-panel.context";
import TouchOpaq from './TouchOpaq';

/**
 * Component that renders the landmark details panel header
 * @param 
 */
export const DetailsHeader: React.FC = observer((props) => {
    const {landmarkOwnedByUser, anonUserId, setAlert} = useAuth()
    const {
        addLandmarkPhotoMutation, 
        deleteLandmarkPhotoMutation, 
        landmarkQuery,
        editLandmarkMutation,
        rateLandmarkMutation} = useSelectedLandmarkContext();

    const photosAreBusy = () => {
        return store.selectedLm.processingPhoto ||
            addLandmarkPhotoMutation.isLoading ||
            deleteLandmarkPhotoMutation.isLoading
    }

    const navigation = useNavigation();

    /**
     * Calls the {@linkcode deleteLandmark} mutation from the {@link useLandmarks} hook and closes the modal once finished.
     */
     const removeLandmark = async () => {
        Alert.alert("Are you sure you want to delete this landmark?", undefined,
      [{ text: "Cancel", }
        ,
      {
        text: "Confirm", onPress: async () => {
            await deleteLandmarkPhotoMutation.mutateAsync(store.selectedLm.selectedLandmarkId);   
            store.selectedLm.close();
            Alert.alert("Landmark Deleted", "This landmark has been deleted.");
        }
      }])
    }

    /**
     * Calls the {@linkcode updateLandmark} mutation from the {@link useLandmarks} hook and closes the modal once finished.
     */
     const editLandmark = async () => {
        if (store.selectedLm.pendingLandmarkData) {
            await editLandmarkMutation.mutateAsync(store.selectedLm.pendingLandmarkData) 
        }
        
        store.selectedLm.setEditing(false);
    }

    /**
     * Calls the {@linkcode rateLandmarkAsunc} mutation from the {@link useLandmarks} hook. If 1, the landmark will be upvoted. If -1, it will be downvoted
     */
     const rateLandmark = async (rating: 1 | -1) => {
        if (landmarkQuery?.data?.landmark) {
            await rateLandmarkMutation.mutateAsync({id: store.selectedLm.selectedLandmarkId, rating: rating});
        }
    }

    const HeaderContent: React.FC = () => {
        // landmark is owned by user
        if (landmarkOwnedByUser(landmarkQuery.data?.landmark)) {
            // editing is enabled
            if (store.selectedLm.editing) {
                return (
                    <View style={{ flexDirection: 'row', justifyContent: 'space-between', alignItems: "center", width: '100%' }}>

                        <TouchOpaq
                            func={() => store.selectedLm.setEditing(false)}
                            size={25}
                            col={"white"}
                            name={"close"}
                        />

                        {store.selectedLm.pendingLandmarkData?.description && store.selectedLm.pendingLandmarkData.landmark_type ? // only display check if fields are populated
                            // <FontAwesome style={{ marginTop: 2 }} size={25} color="white" name="check" onPress={async () => props.editLandmark()} /> 
                            <TouchOpaq
                                func={async () => await editLandmark()}
                                size={25}
                                col="white"
                                name={"check"}
                            />
                            : null}
                    </View>
                )
            }
            // editing is disabled
            else {
                return (

                    <View style={{ borderColor: 'blue', borderWidth: 0, flexDirection: 'row', alignItems: "center", justifyContent: 'space-between', width: '100%' }}>
                        <View style={{ borderColor: 'green', borderWidth: 0, flexDirection: 'row', alignItems: "center", justifyContent: "flex-end" }}>

                            <TouchOpaq
                                func={() => store.selectedLm.setEditing(true)}
                                size={25}
                                col={"white"}
                                name={"edit"}
                            />

                            <TouchOpaq
                                func={() => removeLandmark()}
                                size={25}
                                col={"white"}
                                name={"trash"}
                            />

                            <Text style={{ color: 'white', fontSize: 20, marginTop: 2, marginLeft: 10 }} >{landmarkQuery.data?.landmark?.rating}</Text>
                            {/* <Text>Lil Tjay rn</Text> */}

                            <FontAwesome style={{ marginLeft: 5, marginBottom: 2, marginRight: 10 }} color="white" size={25} name="thumbs-up" />
                        </View>

                        <TouchOpaq
                            func={() => store.selectedLm.close()}
                            size={25}
                            col={"white"}
                            name={"close"}
                        />



                    </View>
                )
            }
        }
        // landmark is not owned by user
        else {
            return (
                <View style={{ flexDirection: 'row', justifyContent: 'space-between', alignItems: "center", width: '100%' }}>
                    {landmarkQuery.data?.ratedByUser ?  // landmark has already been liked by the current user
                        <View style={{ flexDirection: 'row' }} >
                            <Text style={{ color: 'white', fontSize: 20, marginTop: 2 }} >{landmarkQuery.data?.landmark.rating}</Text>
                            <FontAwesome style={{ marginLeft: 5, marginTop: 2, marginRight: 30 }} color="white" size={25} name="thumbs-up" />
                            <Text style={{ color: 'lightgray', fontSize: 20, marginTop: 2, opacity: 0.7 }} >You liked this </Text>
                            <TouchableOpacity style={{ flexDirection: 'row' }} onPress={() => rateLandmark(-1)}>
                                <Text style={{ color: 'white', fontSize: 20, marginTop: 2, opacity: 0.8 }} >(</Text>
                                <Text style={{ color: 'white', fontSize: 20, marginTop: 2, opacity: 0.8, textDecorationLine: "underline" }} >Undo</Text>
                                <Text style={{ color: 'white', fontSize: 20, marginTop: 2, opacity: 0.8 }} >)</Text>
                            </TouchableOpacity>
                        </View> : // landmark has not been liked by user
                        <TouchableOpacity onPress={async () => { //this touchable will add a like to this landmark
                            if (anonUserId) {
                                setAlert({
                                    title: "Authentication required",
                                    type: "warning",
                                    message: "Oops! In order to rate landmarks, you have to log in.",
                                    callback: () => {store.selectedLm.close(); navigation.navigate("Account" as never)},
                                    callbackButtonText: 'Go to login'
                                });
                            }
                            else if (!landmarkOwnedByUser(landmarkQuery.data?.landmark)) {
                                await rateLandmark(1);
                            }
                        }}>
                            <View style={{ flexDirection: 'row' }} >
                                <Text style={{ color: 'white', fontSize: 20, marginTop: 2 }} >{landmarkQuery.data?.landmark?.rating}</Text>
                                <FontAwesome style={{ marginLeft: 5, marginTop: 2 }} color="white" size={25} name="thumbs-up" />
                            </View>
                        </TouchableOpacity>}
                        <TouchOpaq
                            func={() => store.selectedLm.close()}
                            col={"white"}
                            size={25}
                            name={"close"}
                        />

                </View>
            )
        }
    }
    return (
        <View style={styles.detailsHeader}>
            {!photosAreBusy() ?
                <HeaderContent />
                : null}
        </View>
    )
})

const styles = StyleSheet.create({
    detailsHeader: {
        borderBottomWidth: 1,
        borderColor: 'white',
        flexDirection: 'row',
        justifyContent: 'space-between',
        alignItems: 'center',
        backgroundColor: '#df3f3f',
        marginBottom: 10,
        paddingVertical: 10,
        paddingHorizontal: 20
    },
})