import { FontAwesome } from "@expo/vector-icons"
import { ImageInfo } from "expo-image-picker/build/ImagePicker.types"
import { observer } from "mobx-react"
import React, { useState } from "react"
import { ActivityIndicator, Image, ScrollView, Text, TouchableOpacity, View } from "react-native"
import { store } from "../../../../main-store"
import { useAuth } from "../../../../state/external/auth-provider"
import { LMPhoto } from "../../../../types"
import { IconButton, PrimaryButton } from "../../../Buttons"
import { PhotoPicker } from "../../../PhotoPicker"
import { useSelectedLandmarkContext } from "./selected-landmark-panel.context"

export const LandmarkPhotos: React.FC = observer((props) => {
    const {landmarkOwnedByUser} = useAuth();
    const {addLandmarkPhotoMutation, landmarkQuery, deleteLandmarkPhotoMutation} = useSelectedLandmarkContext();

    /**
     * Flag that toggles the photo source menu being displayed
    */
    const photoSourceMenuOpenedState = false;
    const [photoSourceMenuOpened, togglePhotoSourceMenu] = useState<boolean>(photoSourceMenuOpenedState);

    /**
     * Handles selection of a selected photo. Adds the photo data to the database.
    */
     const _onPhotoSelected = async (result: ImageInfo) => {
        togglePhotoSourceMenu(false)
        const photo: LMPhoto = {id: '', image_b64: 'data:image/png;base64,' + result.base64, height: result.height, width: result.width, landmark: landmarkQuery.data?.landmark?.id}
        store.selectedLm.setProcessingPhoto(false)
        await addLandmarkPhotoMutation.mutateAsync(photo);
    }

    /**
     * Sets given image as selected photo to be maximized on the screen
     * @param 
     */
    const maximizePhoto = (i: number): void => {
        if (!store.selectedLm.editing) {
            store.selectedLm.selectImage(i)
        }
        else {
            console.warn('[LandmarkDetails]: Editing is enabled, can\'t maximize image')
        }
    }

    return (
        <>
            {
            deleteLandmarkPhotoMutation.isLoading || 
            deleteLandmarkPhotoMutation.isError || 
            addLandmarkPhotoMutation.isLoading || 
            addLandmarkPhotoMutation.isError ||
            store.selectedLm.processingPhoto ?
            <View style={{justifyContent: "space-evenly", alignItems: "center"}}>
                <Text style={{color: 'white', fontSize: 20, marginBottom: 40}}>{
                    deleteLandmarkPhotoMutation.isLoading || addLandmarkPhotoMutation.isLoading ? "Loading photos" :
                    deleteLandmarkPhotoMutation.isError || addLandmarkPhotoMutation.isError ? "There was an error loading photos" : 
                    store.selectedLm.processingPhoto ? "Processing photo" : null
                }</Text>
                {
                    deleteLandmarkPhotoMutation.isLoading || addLandmarkPhotoMutation.isLoading || store.selectedLm.processingPhoto ? <ActivityIndicator color='white' size="large"/> :
                    deleteLandmarkPhotoMutation.isError || addLandmarkPhotoMutation.isError ? <PrimaryButton text="Okay" style={{borderColor: 'white', borderWidth: 1}} onPress={() => store.selectedLm.close()} /> : null
                }
            </View> : 
            <>
                <PhotoPicker 
                    multiple={true} 
                    menuType="alert" 
                    onReceivedPhotoResult={async result => await _onPhotoSelected(result)} 
                    photoSourceMenuOpened={photoSourceMenuOpened} 
                    onBeforeLaunchPicker={() => store.selectedLm.setProcessingPhoto(true)} 
                    cancel={() => {store.selectedLm.setProcessingPhoto(false) ; togglePhotoSourceMenu(false)}}/> 
                {landmarkQuery.data?.landmark?.photos?.length > 0 ? 
                <>
                    <ScrollView nestedScrollEnabled={true} contentContainerStyle={{alignItems: 'center'}} style={{flexDirection: 'row', marginBottom: 5, alignSelf: 'center'}} horizontal={true}>
                        {landmarkQuery.data?.landmark?.photos?.length < 5 && landmarkOwnedByUser(landmarkQuery.data?.landmark) ? <IconButton style={{alignSelf: 'center', padding: 10, opacity: .5, marginLeft: 10}} color='white' size={30} icon="plus" onPress={() => togglePhotoSourceMenu(true)} />: null}
                        {landmarkQuery.data?.landmark?.photos?.map((photo, i) => {    
                            
                            return (
                                <TouchableOpacity activeOpacity={1} key={i} style={{marginHorizontal: 1, padding: 14, zIndex: 11}} onPress={() => maximizePhoto(i)}>
                                    <Image style={{alignSelf: 'center', height: 300, width: 200}} source={{uri: 'data:image/png;base64,' + photo.image_b64}}/> 
                                    {landmarkOwnedByUser(landmarkQuery.data?.landmark) ? <IconButton icon="times-circle" color="lightgray" style={{position: 'absolute', top: -2, right: 0}} size={25} onPress={() => console.log('delete photo')} /> : null}
                                </TouchableOpacity>
                            )
                        })}
                    </ScrollView>
                    
                </> : 
                <>
                {landmarkOwnedByUser(landmarkQuery.data?.landmark) ?
                <TouchableOpacity style={{marginTop: 30, justifyContent: 'center', alignItems: 'center', opacity: .7}} onPress={() => {togglePhotoSourceMenu(true)}}>
                    <Text style={{fontSize: 20, marginBottom: 10, color: 'white'}}>Add photo of landmark</Text>
                    <FontAwesome name="plus" size={30} color='white' />
                </TouchableOpacity> : null}
                </>}
            </> }
        </>
    )
})