﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { FontAwesome } from "@expo/vector-icons";
import { observer } from "mobx-react";
import React, { useEffect } from "react";
import { ActivityIndicator, Alert, Dimensions, Image, Keyboard, SafeAreaView, ScrollView, StyleSheet, Text, TouchableOpacity, View } from "react-native";
import Modal from 'react-native-modal';
import { store } from "../../../../main-store";
import { useAuth } from "../../../../state/external/auth-provider";
import { colors } from "../../../../utils/GlobalUtils";
import { IconButton, PrimaryButton } from "../../../Buttons";
import { DetailsBody } from "./details-body";
import { DetailsHeader } from "./details-header";
import { useSelectedLandmarkContext } from "./selected-landmark-panel.context";

/**
 * Component that renders the details of a selected {@link Landmark} and allows the user to edit those details. Contained within a [react-native-modal]{@link https://github.com/react-native-modal/react-native-modal}.
 * @component
 * @category Map
 */
export const SelectedLandmarkPanel: React.FC = observer(() => {
    const {landmarkOwnedByUser} = useAuth();
    const {deleteLandmarkPhotoMutation, editLandmarkMutation, deleteLandmarkMutation, mainScrollRef, addCommentMutation, editCommentMutation, deleteCommentMutation, landmarkQuery} = useSelectedLandmarkContext();

    useEffect(() => {
        console.log(landmarkQuery.status)
    }, [landmarkQuery.status]);
    /**
     * Prompts user for a confirmation to delete the photo that they just tried to delete
     */
     const tryDeleteSelectedPhoto = () => {
        const photoId = landmarkQuery?.data?.landmark?.photos[store.selectedLm.selectedImageIndex].id
        Alert.alert(
            'Confirm photo removal', 
            'Are you sure you want to delete this photo?', 
            [
                {text: 'Yes', 
                onPress: async () => {
                    await deleteLandmarkPhotoMutation.mutateAsync(photoId)
                    store.selectedLm.selectImage(-1)
                    await landmarkQuery.refetch()
                }},
                {text: 'No'} 
            ]
        )  
    }

    /**
     * Returns a height for the modal depending on if an image is maximzed, if the keyboard is opened, and if the current landmark has photos associated with it
     */
     const determineModalHeight = () => {
        if (store.selectedLm.selectedImageIndex > -1) 
            return Dimensions.get("window").height 
        else if (store.global.keyboardOpen || store.selectedLm.editing || (!landmarkOwnedByUser(landmarkQuery?.data?.landmark) && landmarkQuery?.data?.landmark?.photos?.length == 0)) {
            return Dimensions.get("window").height * .4
        }
        else if (landmarkQuery?.data?.landmark?.photos?.length > 0) 
            return Dimensions.get("window").height * .9 
        else
            return Dimensions.get("window").height * .6
    }

    return (
        <Modal 
            useNativeDriver={true}
            useNativeDriverForBackdrop={true}
            avoidKeyboard={true}
            onBackdropPress={() => {
                if (store.selectedLm.editing) {
                    Keyboard.dismiss();
                } else {
                    store.selectedLm.close();
                }
            }}
            style={{justifyContent: "flex-end", height: '100%', margin: 0}}
            isVisible={store.selectedLm.panelVisible}>
            <SafeAreaView 
                style={[styles.container, {height: determineModalHeight()}]}>

                {store.selectedLm.selectedImageIndex > -1 ?
                <View style={{ padding: 14}}>
                    <View style={{justifyContent: 'space-between', flexDirection: 'row'}}>
                        <TouchableOpacity onPress={() => store.selectedLm.selectImage(-1)}>
                            <>
                                <Text style={{color: 'white', fontSize:20}}><FontAwesome name="arrow-left" color="white" size={20}/> Back</Text>
                            </>
                        </TouchableOpacity>
                        <IconButton 
                            style={{alignSelf: 'flex-end', marginBottom: 20}} 
                            icon="trash" 
                            color="white" 
                            size={25} 
                            onPress={() => tryDeleteSelectedPhoto()}/>
                    </View>
                    <ScrollView style={{width: '100%'}} ref={mainScrollRef}>
                        <Image style={{resizeMode: 'contain', alignSelf: 'center', height: Dimensions.get('window').height * .9, width: Dimensions.get('window').width}} source={{uri: 'data:image/png;base64,' + landmarkQuery?.data?.landmark?.photos[store.selectedLm.selectedImageIndex].image_b64}}/> 
                    </ScrollView>
                </View>
                :
                (landmarkQuery.isSuccess || landmarkQuery.isIdle) && 
                (editLandmarkMutation.isIdle || editLandmarkMutation.isSuccess) && 
                (deleteLandmarkMutation.isIdle || deleteLandmarkMutation.isSuccess) ?
                <>
                    <DetailsHeader/>
                    <DetailsBody/>
                </> :
                <View style={{height: '100%', justifyContent: "space-evenly", alignItems: "center", marginHorizontal: 20}}>
                    <Text style={{color: 'white', fontSize: 20}}>{
                        landmarkQuery.isLoading ? "Loading landmark..." : 
                        landmarkQuery.isError ? "Something went wrong trying to load the landmark" :
                        deleteLandmarkMutation.isLoading ? "Deleting landmark..." : 
                        deleteLandmarkMutation.isError ? "Something went wrong trying to delete the landmark" :
                        editLandmarkMutation.isLoading ? "Updating landmark..." : 
                        editLandmarkMutation.isError ? "Something went wrong trying to update the landmark" :
                        addCommentMutation.isLoading ? "Adding comment..." : 
                        addCommentMutation.isError ? "Something went wrong trying to add the comment" : 
                        editCommentMutation.isLoading ? "Updating comment..." : 
                        editCommentMutation.isError ? "Something went wrong trying to update the comment" :  
                        deleteCommentMutation.isLoading ? "Deleting comment..." : 
                        deleteCommentMutation.error ? "Something went wrong trying to delete the comment" : null} 
                    </Text>
                    {
                        deleteLandmarkMutation.isLoading || editLandmarkMutation.isLoading || landmarkQuery.isLoading ? <ActivityIndicator color='white' size="large"/> :
                        deleteLandmarkMutation.isError || editCommentMutation.isLoading || landmarkQuery.isError ? <PrimaryButton text="Okay" style={{borderColor: 'white', borderWidth: 1}} onPress={async () => await landmarkQuery.refetch()} /> : null
                    }
                </View>}
            </SafeAreaView>
        </Modal>
    )
})

const styles = StyleSheet.create({
    container: {
        backgroundColor: colors.red,
    },
    detailsContainer: {
        flex: 1,
        marginHorizontal: 20
    },

    commentContainer: {
        flex: 5,
        marginBottom: 40,
    },
    title: {

    },
    input: {
        padding: 5,
        color: 'black'
    }
})