import React from 'react';
import { createContext, useContext, useEffect, useMemo, useRef, useState } from "react";
import { Dimensions, FlatList, Keyboard, ScrollView } from "react-native";
import { UseMutationResult, UseQueryResult } from "react-query";
import { useAddComment, useDeleteComment, useEditComment, useLandmarkComments } from "../../../../api/comments";
import { useDeleteLandmark, useEditLandmark, useRateLandmark } from "../../../../api/landmarks";
import { useAddLandmarkPhoto, useDeleteLandmarkPhoto } from "../../../../api/lm-photos.ts";
import { useOwnedProfile } from "../../../../api/profile/profiles.get-own";
import { store } from "../../../../main-store";
import { useAuth } from "../../../../state/external/auth-provider";
import { Landmark, LMComment, LMPhoto } from "../../../../types";

interface SelectedLandmarkMutationContextState {
    mainScrollRef: React.MutableRefObject<ScrollView | null>;
    commentListRef: React.MutableRefObject<FlatList | null>;
    landmarkQuery: UseQueryResult<{landmark: Landmark, ratedByUser: boolean}, Error>;
    editLandmarkMutation: UseMutationResult<any, unknown, Landmark, unknown>;
    deleteLandmarkMutation: UseMutationResult<any, unknown, string, unknown>;
    addLandmarkPhotoMutation: UseMutationResult<LMPhoto>;
    deleteLandmarkPhotoMutation: UseMutationResult<string>;
    rateLandmarkMutation: UseMutationResult<any, unknown, {id: string;rating: 1 | -1;}, unknown>;
    commentsQuery: UseQueryResult<LMComment[], Error>;
    addCommentMutation: UseMutationResult<void, unknown, LMComment, unknown>;
    editCommentMutation: UseMutationResult<any, unknown, LMComment, unknown>
    deleteCommentMutation: UseMutationResult<any, unknown, string, unknown>;
}

const SelectedLandmarkContext = createContext<SelectedLandmarkMutationContextState>(null);

export const SelectedLandmarkPanelCtxProvider: React.FC = ({children}) => {
    const editLandmarkMutation = useEditLandmark();
    const rateLandmarkMutation = useRateLandmark();
    const deleteLandmarkMutation = useDeleteLandmark();
    const commentsQuery = useLandmarkComments(store.selectedLm.selectedLandmarkId);
    const addCommentMutation = useAddComment();
    const editCommentMutation = useEditComment();
    const deleteCommentMutation = useDeleteComment();
    const addLandmarkPhotoMutation = useAddLandmarkPhoto();
    const deleteLandmarkPhotoMutation = useDeleteLandmarkPhoto();
    
    const {userId, landmarkOwnedByUser} = useAuth()

    // /**
    //  * Holds the state of the {@link Landmark} being displayed.
    //  */
    // const selectedLandmarkState = undefined;
    // const [selectedLandmark, setLandmark] = useState<Landmark | undefined>(selectedLandmarkState);
    /**
     * Holds state of a {@link Landmark} object parallel to {@linkcode selectedLandmarkState} that is manipulated when {@linkcode editingEnabled} is true.
     */
    const [updatedLandmark, setUpdatedLandmark] = useState<Landmark | undefined>(undefined);
    /**
     * Holds text of the current {@link LMComment} being added to this {@link Landmark}.
     */
     const [newCommentId, setNewComment] = useState<string>("");
    /**
     * Holds id of the focused {@link LMComment} in the comment list.
     */
    const [focusedCommentId, setFocusedComment] = useState<string>("");
    /**
     * Holds state of the {@link LMComment} being currently edited.
     */
    const [commentBeingEdited, setCommentBeingEdited] = useState<LMComment | undefined>(undefined);

    const { profile } = useOwnedProfile()

    /**
     * Holds a reference to the Flatlist containing the comments.
     */
    const commentListRef = useRef<FlatList>();
    const mainScrollRef = useRef<ScrollView>();

    useEffect(() => {
        /**
         * Resets the {@linkcode rateLandmark} mutation on successful update.
         * Embedded in a useEffect that listens to the {@linkcode rateLandmarkStatus} value from the {@link useLandmarks} hook.
         * @memberOf LandmarkDetails
         */
         const resetUpdateLandmark = async () => {
            if (rateLandmarkMutation.isSuccess) {
                rateLandmarkMutation.reset()
                await landmarkQuery.refetch()
            }

            if (editLandmarkMutation.isSuccess) {
                editLandmarkMutation.reset()
                await landmarkQuery.refetch()
            }
        }
        resetUpdateLandmark();
    }, [rateLandmarkMutation.status, editLandmarkMutation.status]);

    useEffect(() => {
        /**
         * Resets the {@linkcode updateComment} mutation on successful update.
         * Embedded in a useEffect that listens to the {@linkcode updateCommentStatus} value from the {@link useComments} hook.
         * @memberOf LandmarkDetails
         */
         const resetUpdateComment = () => {
            if (editCommentMutation.isSuccess) {
                commentListRef.current?.scrollToItem({animated: true, item: commentBeingEdited});
                setCommentBeingEdited(undefined)
                Keyboard.dismiss();
            }
        }
        resetUpdateComment();
    }, [editCommentMutation.status]);

    useEffect(() => {
        /**
         * Resets the {@linkcode deleteLandmark} mutation on successful delete.
         * Embedded in a useEffect that listens to the {@linkcode deleteLandmarkStatus} value from the {@link useLandmarks} hook.
         * @memberOf LandmarkDetails
         */
        const resetDeleteLMOnSuccess = () => {
            if (deleteLandmarkMutation.isSuccess) {
                deleteLandmarkMutation.reset()
            }

            console.log(deleteLandmarkMutation.status)
        }
        resetDeleteLMOnSuccess();
    }, [deleteLandmarkMutation.status]);

    useEffect(() => {
        /**
         * Resets the {@link addCommentAsync} mutation on successful add.
         * Embedded in a useEffect that listens to the {@link addCommentAsync} value from the {@link useComment} hook.
         * @memberOf LandmarkDetailsLandmark
         */
        const resetAddCommentOnSuccess = () => {
            if (addCommentMutation.isSuccess) {
                addCommentMutation.reset()
            }
            setNewComment('')
            commentListRef.current?.scrollToIndex({animated: true, index: 0});
            Keyboard.dismiss();
        }
        resetAddCommentOnSuccess();
    }, [addCommentMutation.status]);

    useEffect(() => {
        /**
         * Clears selected comment when {@linkcode newCommentId} state changes.
         * @memberOf LandmarkDetails
         */
        const clearSelectedOnNewCommentChange = () => {
            setFocusedComment('')
        }
        clearSelectedOnNewCommentChange();
    }, [newCommentId]);

    

    const state = useMemo<SelectedLandmarkMutationContextState>(() => {
        return {
            commentListRef,
            mainScrollRef,
            landmarkQuery,
            editLandmarkMutation,
            deleteLandmarkMutation,
            addLandmarkPhotoMutation,
            deleteLandmarkPhotoMutation,
            rateLandmarkMutation,
            commentsQuery,
            addCommentMutation,
            editCommentMutation,
            deleteCommentMutation,
        }
    }, [store.selectedLm.selectedLandmarkId]);

    /**
     * Calls the {@linkcode addComment} mutation from the {@link useComments} hook.
     */
    const addComment = async () => {
        if (newCommentId) {
            await addCommentMutation.mutateAsync({
                edited: false,
                content: newCommentId,
                landmark: store.selectedLm.selectedLandmarkId,
                poster: userId,
                poster_name: profile.username,
                id: ''
            });
        }
    }

    /**
     * Set a comment to be focused in the comment box. It will expand and show delete and edit controls for that comment. 
     * If the comment id given is already selected, the comment will be unselected
     */
    const focusComment = (id: string) => {
        if (focusedCommentId == id) {
            setFocusedComment("");
        }
        else {
            setFocusedComment(id);
        }
    }

    

    /**
     * Calls the {@linkcode editComment} mutation from the {@link useComments} hook.
     */
    const editComment = async (comment: LMComment) => {
        await editCommentMutation.mutateAsync(comment);
    }

    /**
     * Calls the {@linkcode deleteComment} mutation from the {@link useComments} hook.
     */
    const deleteComment = async (id: string) => {
        await deleteCommentMutation.mutateAsync(id);
    }

    const getWindowWidth = () => {
        return Dimensions.get('window').width
    }


    return (
        <SelectedLandmarkContext.Provider value={state}>
            {children}
        </SelectedLandmarkContext.Provider>
    )
}

export const useSelectedLandmarkContext = () => {
    const context = useContext(SelectedLandmarkContext);
    if (!context) {
        throw new Error('useSelectedLandmarkContext must be used within a SelectedLandmarkContextProvider');
    }
    return context;
}